# This is a test for the packet sniffer application...


# For Linux!

import html
import socket, sys
import base64
from prettytable import PrettyTable
from struct import *
from scapy.all import *

# Convert 6 characters of the ethernet address into a hex string
def eth_addr (a) :
    b = "%0.2x:%0.2x:%0.2x:%0.2x:%0.2x:%0.2x" % (a[0], a[1], a[2], a[3], a[4], a[5])
    return b

# Use application INET to create a socket to application
# Also not adding some error correction
try:
    s = socket.socket(socket.AF_PACKET, socket.SOCK_RAW, socket.ntohs(0x0003))

# If socket cannot be created display the error
except (socket.error, msg):

    # Create a string with the error code as msg 0 then the typed message as msg 1 as the second variable
    print('Socket failed creation. Here is the error code: ' + str(msg[0]) + ' Message ' + msg[1])
    sys.exit()
# As long as is true
while 1 == 1:
   
    # Define the packet
    packet = s.recvfrom(65565)

    # Form a differnt packet from with tuple included
    packet = packet[0]

    # Get ethernet header
    eth_length = 14

    eth_header = packet[:eth_length]
    eth = unpack('!6s6sH', eth_header)
    eth_protocol = socket.ntohs(eth[2])
    
    # List information in a table
    x = PrettyTable()

    x.field_names = ["Destination MAC", "Source MAC", "Protocol"]
    
    # List rows for table
    x.add_row([eth_addr(packet[0:6]), eth_addr(packet[6:12]), str(eth_protocol)])

    print(x.get_string(title="IP Packet"))

    # Parse the packets at 8 protocols
    if eth_protocol == 8:
        # Parse the IP header
        # Grab the fist 20 Characters from the ip header
        ip_header = packet[eth_length:20+eth_length]

        # Unpack the ip header the variable above
        iph = unpack('!BBHHHBBH4s4s', ip_header)

        version_ihl = iph[0]
        version = (version_ihl >> 4)
        ihl = (version_ihl & 0xF)

        iph_length = (ihl * 4)

        ttl = iph[5]
        protocol = iph[6]
        s_addr = socket.inet_ntoa(iph[8]);
        d_addr = socket.inet_ntoa(iph[9]);

        # List more information into a table
        y = PrettyTable()

        y.field_names = ["Version", "IP Header Length", "TTL", "Protocol", "Source Address", "Destination Address"]

        # List row information
        y.add_row([str(version), str(ihl), str(ttl), str(protocol), str(s_addr), str(d_addr)])

        print(y.get_string(title="Ethernet Packet Header"))

        if (protocol == 6):
            t = iph_length + eth_length
            tcp_header = packet[t:t+20]

            # Now unpack then again
            tcph = unpack('!HHLLBBHHH', tcp_header)

            source_port = tcph[0]
            dest_port = tcph[1]
            sequence = tcph[2]
            acknowledgment = tcph[3]
            doff_reserved = tcph[4]
            tcph_length = (doff_reserved >> 4)

            # Print more stuff into tables!
            x = PrettyTable()
            
            x.field_names = ["Source Port", "Dest Port", "Sequence Number", "Acknowledgement", "TCP Header Length"]

            # Add information as rows.. This is going to get redundent lmao
            x.add_row([str(source_port), str(dest_port), str(sequence), str(acknowledgment), str(tcph_length)])

            print(x)

            h_size = iph_length + tcph_length * 4
            data_size = (len(packet) - h_size)

            # Receive data from the packet
            data = packet[h_size:]

            # Print data in the table
            print()
            print("PACKET INFORMATION:", data.decode(errors="ignore"))
            print()

        # ICMP Packets
        elif (protocol == 1):
            u = iph_length + eth_length
            icmph_length = 4
            icmp_header = packet[u:u+4]
            
            # Unpack ICMP Packets
            icmph = unpack('!BBH', icmp_header)

            icmp_type = icmph[0]
            code = icmph[1]
            checksum = icmph[2]

            print('Type: ' + str(icmp_type) + ' Code: ' + str(code) + ' Checksum: ' + str(checksum))

            h_size = eth_length + iph_length + icmph_length
            data_size = len(packet) - h_size

            # Retrieve the data from the packet
            data = packet[h_size:]

            print()
            print('Data: ', data.decode(errors="ignore"))
            print()

        # UDP Packets
        elif (protocol == 17):
            u = iph_length + eth_length
            udph_length = 8
            udp_header = packet[u:u+8]

            # Proceed to unpacking
            udph = unpack('!HHHH', udp_header)

            source_port = udph[0]
            dest_port = udph[1]
            length = udph[2]
            checksum = udph[3]

            print('Source Port: ' + str(source_port) + ' Dest_port' + str(dest_port) + ' Length: ' + str(length) + ' Checksum: ' + str(checksum))

            h_size = (eth_length + iph_length + udph_length)
            data_size = len(packet) - h_size

            # Retrieve data from packet
            data = packet[h_size:]

            print()
            print('Data: ', data.decode(errors="ignore"))
            print()

    else:
        print('Protocol other than TCP/UDP/ICMP')
        print()
































